import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Form, Row, Col, notification, Tooltip, Rate } from 'antd'
import { QuestionCircleOutlined, StarFilled } from '@ant-design/icons'
import moment from 'moment'

import Api from '@/api'
import options from '@/store/options.js'
import { formRule } from '@/utils/option.js'
import Utils from '@/utils/utils.js'
import asyncComponent from '@/utils/asyncComponent'
import MKEmitter from '@/utils/events.js'
import MKInput from './mkInput'
import MKNumberInput from './mkNumberInput'
import MKSelect from './mkSelect'
import MkIcon from '@/components/mk-icon'
import './index.scss'

const MKCheckCard = asyncComponent(() => import('./mkCheckCard'))
const MKSwitch = asyncComponent(() => import('./mkSwitch'))
const MKCheckbox = asyncComponent(() => import('./mkCheckbox'))
const MKRadio = asyncComponent(() => import('./mkRadio'))
const MKDatePicker = asyncComponent(() => import('./mkDatePicker'))
const MKTextArea = asyncComponent(() => import('./mkTextArea'))
const MKFileUpload = asyncComponent(() => import('../fileupload'))
const MKColor = asyncComponent(() => import('./mkColor'))
const MkFormula = asyncComponent(() => import('./mkFormula'))
const MkCascader = asyncComponent(() => import('./mkCascader'))
const MKEditor = asyncComponent(() => import('@/components/editor'))

class MainSearch extends Component {
  static propTpyes = {
    action: PropTypes.object,    // 按钮信息、表单列表
    data: PropTypes.any,         // 表格数据
    BID: PropTypes.any,          // 主表ID
    BData: PropTypes.any,        // 主表数据
    inputSubmit: PropTypes.func  // input回车提交
  }

  state = {
    formlist: [],    // 表单项
  }

  record = {}

  componentDidMount () {
    const { data, BData, action } = this.props

    let linkFields = {} // 关联菜单
    let controlFields = {} // 控制表单
    let deForms = []    // 需要动态获取下拉菜单的表单

    let record = {}
    let formlist = fromJS(action.fields).toJS()
    let fieldMap = new Map()
    let check = action.setting.formType === 'check'

    formlist = formlist.filter(item => {
      if (item.supField) { // 多层表单控制
        let supvals = []
        if (item.supvalue) {
          item.supvalue.split(',').forEach(val => {
            supvals.push(val)
            if (/^([-]?(0|[1-9][0-9]*)(\.[0-9]+)?)$/.test(val)) {
              supvals.push(+val)
            }
          })
        } else {
          supvals.push('')
        }
        controlFields[item.supField] = controlFields[item.supField] || []
        controlFields[item.supField].push({field: item.field, values: supvals})
      }
      // if (item.type === 'link') {
      if (item.linkField) {
        linkFields[item.linkField] = linkFields[item.linkField] || []
        linkFields[item.linkField].push({field: item.field, uuid: item.uuid})
      }

      if (item.style) {
        delete item.style.marginTop
        delete item.style.marginBottom
        delete item.style.marginLeft
        delete item.style.marginRight
      }

      if (item.type === 'split' || item.type === 'formula') return true
      if (item.type === 'hint') {
        if (item.field && data && data[item.field]) {
          item.message = data[item.field]
        }
        delete item.field
        return true
      } else if (item.type === 'date') { // 时间搜索
        item.precision = item.precision || 'day'
      } else if (item.type === 'datetime') {
        item.type = 'date'
        item.precision = 'second'
      }

      if (!item.field || !['text', 'number', 'switch', 'rate', 'select', 'link', 'cascader', 'linkMain', 'funcvar', 'date', 'datemonth', 'radio', 'checkbox', 'checkcard', 'fileupload', 'textarea', 'multiselect', 'brafteditor', 'color'].includes(item.type)) return false

      if (/^\s+$/.test(item.label)) {
        item.style = item.style || {}
        item.style.color = 'transparent'
      }

      // 数据自动填充
      let readin = item.readin !== 'false'
      item.readin = item.readin !== 'false' && item.readin !== 'top'
      item.readonly = check || item.readonly === 'true'
      item.writein = item.writein !== 'false'
      item.hidden = item.hidden === 'true'
      item.fieldlength = item.fieldlength || 50

      if (item.type === 'funcvar') {
        readin = false
        item.readin = false
        item.hidden = true
      } else if (item.type === 'number') {
        item.decimal = item.decimal || 0
        item.fieldlength = item.decimal
        item.initval = item.initval || 0
      }

      if (['select', 'link', 'radio', 'checkbox', 'checkcard', 'multiselect', 'cascader'].includes(item.type)) {
        item.options = item.options || []
        item.options = item.options.filter(cell => {
          cell.value = cell.Value
          cell.label = cell.Text

          return !cell.Hide
        })
        if (item.setAll === 'true' && ['select', 'link', 'radio'].includes(item.type)) { // 添加空值
          item.options.unshift({
            key: Utils.getuuid(),
            value: '',
            label: item.emptyText || '空',
            ParentID: ''
          })
        }
        item.oriOptions = fromJS(item.options).toJS()
      }

      let newval = '$empty'

      if (readin && data && data.hasOwnProperty(item.field)) {
        newval = data[item.field]
      }
      if (item.type === 'linkMain') {
        newval = BData && BData[item.field] ? BData[item.field] : '$empty'
      } else if (item.type === 'date') { // 时间搜索
        let format = 'YYYY-MM-DD'
        let _format = 'YYYY-MM-DD HH:mm:ss'
        if (item.precision === 'day') {
          _format = 'YYYY-MM-DD'
        } else if (item.precision === 'hour') {
          format = 'YYYY-MM-DD HH'
        } else if (item.precision === 'minute') {
          format = 'YYYY-MM-DD HH:mm'
        } else if (item.precision === 'second') {
          format = 'YYYY-MM-DD HH:mm:ss'
        }

        if (newval !== '$empty') {
          newval = moment(newval, format).format(_format)
          newval = newval === 'Invalid date' ? '$empty' : newval
        }
        if (newval === '$empty' && item.initval) {
          newval = moment().subtract(item.initval, 'days').format(_format)
        }
      } else if (item.type === 'datemonth') {
        if (newval !== '$empty') {
          newval = moment(newval, 'YYYY-MM').format('YYYY-MM')
          newval = newval === 'Invalid date' ? '$empty' : newval
        }
        if (newval === '$empty' && item.initval) {
          newval = moment().subtract(item.initval, 'month').format('YYYY-MM')
        }
      // } else if (item.type === 'datetime') {
      //   if (newval !== '$empty') {
      //     newval = moment(newval, 'YYYY-MM-DD HH:mm:ss').format('YYYY-MM-DD HH:mm:ss')
      //     newval = newval === 'Invalid date' ? '$empty' : newval
      //   }
      //   if (newval === '$empty' && item.initval) {
      //     if (item.initval === '0') {
      //       newval = moment().format('YYYY-MM-DD HH:mm:ss')
      //     } else {
      //       newval = moment().subtract(item.initval, 'days').format('YYYY-MM-DD') + ' 00:00:00'
      //     }
      //   }
      } else if (item.type === 'switch') { // 开关只接收固定值
        if (newval !== '$empty' && (newval === item.closeVal || newval === item.openVal)) {

        } else if (item.initval === true) {
          newval = item.openVal
        } else {
          newval = item.closeVal
        }
      }

      if (newval !== '$empty') {
        item.initval = newval
      } else if (item.initval === undefined) {
        item.initval = ''
      }

      if (['select', 'link', 'radio', 'checkbox', 'checkcard', 'multiselect', 'cascader'].includes(item.type) && item.resourceType === '1') {
        deForms.push(item)
      } else if (item.type === 'rate') {
        item.rateCount = item.rateCount || 5
        item.allowHalf = item.allowHalf === 'true'

        if (item.allowHalf) {
          item.initval = parseFloat(item.initval)
          if (item.initval % 0.5 !== 0) {
            item.initval = parseInt(item.initval)
          }
        } else {
          item.initval = parseInt(item.initval)
        }

        if (isNaN(item.initval) || item.initval < 0) {
          item.initval = 0
        } else if (item.initval > item.rateCount) {
          item.initval = item.rateCount
        }
      }

      if (item.type === 'text') {
        if (typeof(item.initval) === 'number') {
          item.initval = item.initval + ''
        }
        let _rules = [{
          pattern: /^[^']*$/ig,
          message: '不可使用英文状态的单引号！'
        }, {
          validator: (rule, value, callback) => {
            if (/--/ig.test(value)) {
              callback('不可使用 -- ！')
            } else {
              callback()
            }
          }
        }, {
          required: item.required === 'true',
          message: item.label + '不可为空!'
        }]

        if (!item.lenControl || item.lenControl === 'limit') {
          _rules.push({
            max: item.fieldlength,
            message: formRule.input.formMessage.replace('@max', item.fieldlength)
          })
        }

        if (item.regular) {
          if (item.regular === 'number') {
            _rules.push({
              pattern: /^[0-9.-]*$/ig,
              message: formRule.input.numbermsg
            })
          } else if (item.regular === 'letter') {
            _rules.push({
              pattern: /^[a-zA-Z]*$/ig,
              message: formRule.input.lettermsg
            })
          } else if (item.regular === 'letter&number') {
            _rules.push({
              pattern: /^[a-zA-Z0-9@_.]*$/ig,
              message: '请输入数字、字母以及@_.'
            })
          } else if (item.regular === 'phone') {
            _rules.push({
              pattern: /^(13[0-9]|14[01456879]|15[0-35-9]|16[2567]|17[0-8]|18[0-9]|19[0-35-9])\d{8}$/ig,
              message: '请正确输入手机号'
            })
          } else if (item.regular === 'email') {
            _rules.push({
              pattern: /^([a-zA-Z0-9_-])+@([a-zA-Z0-9_-])+(\.[a-zA-Z0-9_-])+/,
              message: '请正确输入邮箱地址'
            })
          } else if (item.regular === 'funcname') {
            _rules.push({
              pattern: /^[\u4E00-\u9FA50-9a-zA-Z_]*$/ig,
              message: formRule.input.funcname
            })
          }
        }
        
        item.rules = _rules
      } else if (item.type === 'number') {
        item.rules = [{
          required: true,
          message: item.label + '不可为空!'
        }, {
          validator: (rule, value, callback) => this.handleConfirmPassword(rule, value, callback, item)
        }]
      } else if (item.type === 'textarea') {
        let _rules = [
          {
            required: item.required === 'true',
            message: item.label + '不可为空!'
          },
          {
            max: item.fieldlength,
            message: formRule.input.formMessage.replace('@max', item.fieldlength)
          },
          {
            pattern: /^[^']*$/ig,
            message: '不可使用英文状态的单引号！'
          }, {
            validator: (rule, value, callback) => {
              if (/--/ig.test(value)) {
                callback('不可使用 -- ！')
              } else {
                callback()
              }
            }
          }
        ]
        item.rules = _rules
      } else if (item.type === 'brafteditor') {
        item.rules = [
          {
            required: item.required === 'true',
            message: item.label + '不可为空!'
          },
          {
            max: item.fieldlength,
            message: formRule.input.formMessage.replace('@max', item.fieldlength)
          }
        ]
      } else if (item.type === 'linkMain') {
        item.rules = [
          {
            required: item.required === 'true',
            message: item.label + '不可为空!'
          }
        ]
      } else {
        item.rules = [
          {
            required: item.required === 'true',
            message: '请选择' + item.label + '!'
          }
        ]
      }

      fieldMap.set(item.field, item)

      return true
    })

    Object.keys(controlFields).forEach(key => {
      if (!fieldMap.has(key)) return

      let supItem = fieldMap.get(key)
      let fields = []
      
      controlFields[key].forEach(item => {
        if (!fieldMap.has(item.field)) return

        let cell = fieldMap.get(item.field)

        if (cell.hidden) return

        if (supItem.hidden || !item.values.includes(supItem.initval)) {
          cell.hidden = true
          fieldMap.set(item.field, cell)
        }

        fields.push(item)
      })

      supItem.controlFields = fields
      
      fieldMap.set(key, supItem)
    })

    let reFieldsVal = null

    formlist = formlist.map(cell => {
      if (cell.labelwidth) {
        cell.labelCol = {style: {width: cell.labelwidth + '%'}}
        cell.wrapperCol = {style: {width: (100 - cell.labelwidth) + '%'}}
      }
      if (!cell.field || !fieldMap.has(cell.field)) return cell
      let item = fieldMap.get(cell.field)

      // 下级表单控制-字段写入
      if ((['select', 'radio', 'link'].includes(item.type) || (item.type === 'checkcard' && item.multiple !== 'true') || (item.type === 'cascader' && item.resourceType !== '2')) && item.linkSubField) {
        item.subFields = []
        item.linkSubField.forEach(m => {
          let n = fieldMap.get(m)
          if (n && ['text', 'number', 'textarea'].includes(n.type)) {
            item.subFields.push({
              uuid: n.uuid,
              field: m
            })
          }
        })

        if (item.subFields.length === 0) {
          item.subFields = null
        } else if (item.oriOptions.length > 0) {
          item.oriOptions = item.oriOptions.map(cell => {
            item.subFields.forEach(m => {
              cell[m.field] = cell[m.field] === undefined ? '' : cell[m.field]
            })
            return cell
          })
          
          item.options = fromJS(item.oriOptions).toJS()
        }
        item.linkSubField = null
      }

      // if (item.type === 'link') {
      if (item.linkField) {
        item.supInitVal = ''

        if (fieldMap.has(item.linkField)) {
          item.supInitVal = fieldMap.get(item.linkField).initval || ''
        } else if (data && data.hasOwnProperty(item.linkField)) {
          item.supInitVal = data[item.linkField]
        }
        
        item.options = item.oriOptions.filter(option => option.ParentID === item.supInitVal || option.value === '')
      }

      if (['select', 'link', 'radio'].includes(item.type) && item.resourceType === '0') { // 选中第一项
        if (typeof(item.initval) === 'string' && item.initval.indexOf('$first') > -1) {
          item.initval = item.options[0] ? item.options[0].value : ''
        }
      }

      if (typeof(item.initval) === 'string' && item.initval.indexOf('$first') > -1) {
        record[item.field] = ''
      } else {
        record[item.field] = item.initval
      }

      if (linkFields[item.field]) {
        item.linkFields = linkFields[item.field]
      }
      
      if (item.enter === 'tab' || item.enter === 'sub') {
        if (fieldMap.has(item.tabField)) {
          item.tabUuid = fieldMap.get(item.tabField).uuid
        } else if (item.enter === 'tab') {
          item.enter = 'false'
        } else if (item.enter === 'sub') {
          item.tabUuid = item.uuid
        }
      }

      if (item.subFields && item.options.length > 0) {
        let initval = item.initval
        if (item.type === 'cascader' && item.separator) {
          initval = initval.split(item.separator).pop()
        }
        // eslint-disable-next-line
        let option = item.options.filter(cell => initval == cell.value)[0]

        if (option) {
          reFieldsVal = reFieldsVal || {}
          item.subFields.forEach(n => {
            reFieldsVal[n.field] = option[n.field]
          })
        }
      }
      
      return item
    })

    if (reFieldsVal) {
      formlist = formlist.map(cell => {
        if (reFieldsVal[cell.field] === undefined) return cell

        cell.initval = reFieldsVal[cell.field]
        record[cell.field] = reFieldsVal[cell.field]

        return cell
      })
    }

    this.record = record

    this.setState({ formlist }, () => {
      if (action.setting && action.setting.focus && fieldMap.has(action.setting.focus)) {
        setTimeout(() => {
          MKEmitter.emit('mkFC', 'focus', fieldMap.get(action.setting.focus).uuid)
        }, 500)
      }

      if (deForms.length > 0) {
        if (!window.GLOB.mkHS && options.sysType === 'local' && window.GLOB.systemType !== 'production') {
          this.improveSimpleActionForm(deForms)
        } else {
          this.improveActionForm(deForms)
        }
      }
    })
  }

  /**
   * @description 获取下拉表单选项信息
   */
  improveActionForm = (deForms) => {
    const { BID, action } = this.props

    let deffers = []
    let mainItems = []  // 云端或单点数据
    let localItems = [] // 本地数据
    let cache = action.setting.cache !== 'false'

    deForms.forEach(item => {
      if (item.database === 'sso') {
        mainItems.push(`select '${item.field}' as obj_name,'${item.arr_field}' as arr_field,'${item.base_sql}' as LText`)
      } else {
        localItems.push(`select '${item.field}' as obj_name,'${item.arr_field}' as arr_field,'${item.base_sql}' as LText`)
      }
    })
    
    if (!window.GLOB.mkHS && options.sysType !== 'local') {
      localItems = [...localItems, ...mainItems]
      mainItems = []
    }

    // 本地请求
    let param = {
      func: 'sPC_Get_SelectedList',
      LText: localItems.join(' union all '),
      obj_name: '',
      arr_field: '',
      table_type: 'Y',
      BID: BID || ''
    }

    if (param.LText) {
      param.LText = Utils.formatOptions(param.LText)
      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt(param.LText, param.timestamp)

      if (window.GLOB.mkHS) { // 云端数据验证
        param.open_key = Utils.encryptOpenKey(param.secretkey, param.timestamp)
      }

      deffers.push(
        new Promise(resolve => {
          Api.getSystemCacheConfig(param, cache).then(res => {
            if (!res.status) {
              notification.warning({
                top: 92,
                message: res.message,
                duration: 5
              })
            }
            resolve(res)
          })
        })
      )
    }

    // 系统请求
    let mainparam = {
      func: 'sPC_Get_SelectedList',
      LText: mainItems.join(' union all '),
      obj_name: '',
      arr_field: '',
      table_type: 'Y',
      BID: BID || ''
    }

    if (mainparam.LText) {
      mainparam.LText = Utils.formatOptions(mainparam.LText)
      mainparam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      mainparam.secretkey = Utils.encrypt(mainparam.LText, mainparam.timestamp)

      if (window.GLOB.mkHS) { // 云端数据验证
        mainparam.open_key = Utils.encryptOpenKey(mainparam.secretkey, mainparam.timestamp)
        if (options.cloudServiceApi) {
          mainparam.rduri = options.cloudServiceApi
          mainparam.userid = sessionStorage.getItem('CloudUserID') || ''
          mainparam.LoginUID = sessionStorage.getItem('CloudLoginUID') || ''
        }
      } else if (window.GLOB.mainSystemApi) {
        mainparam.rduri = window.GLOB.mainSystemApi
      }

      deffers.push(
        new Promise(resolve => {
          Api.getSystemCacheConfig(mainparam, cache).then(res => {
            if (!res.status) {
              notification.warning({
                top: 92,
                message: res.message,
                duration: 5
              })
            }
            resolve(res)
          })
        })
      )
    }

    Promise.all(deffers).then(response => {
      let result = {...response[0], ...(response[1] || {})}

      delete result.ErrCode
      delete result.ErrMesg
      delete result.message
      delete result.status

      this.resetFormList(result)
    })
  }

  /**
   * @description 测试系统获取下拉表单选项信息
   */
  improveSimpleActionForm = (deForms) => {
    let cache = this.props.action.setting.cache !== 'false'

    let deffers = deForms.map((form, index) => {
      let param = {
        func: 'sPC_Get_SelectedList',
        LText: form.data_sql,
        obj_name: form.field,
        arr_field: form.arr_field,
        BID: this.props.BID || ''
      }
  
      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt(param.LText, param.timestamp)
  
      return (
        new Promise(resolve => {
          setTimeout(() => {
            Api.getSystemCacheConfig(param, cache).then(res => {
              if (!res.status) {
                notification.warning({
                  top: 92,
                  message: res.message,
                  duration: 5
                })
              }
              resolve(res)
            })
          }, index * 30)
        })
      )
    })

    Promise.all(deffers).then(response => {
      let result = {}
      response.forEach(res => {
        result = {...result, ...res}
      })
      
      delete result.ErrCode
      delete result.ErrMesg
      delete result.message
      delete result.status

      this.resetFormList(result)
    })
  }

  resetFormList = (result) => {
    let reFieldsVal = null
    let _formlist = fromJS(this.state.formlist).toJS().map(item => {
      if (['select', 'link', 'radio', 'checkbox', 'checkcard', 'multiselect', 'cascader'].includes(item.type) && result[item.field] && result[item.field].length > 0) {
        let options = []
        result[item.field].forEach(cell => {
          let _cell = { key: Utils.getuuid() }

          if (item.type !== 'checkcard') {
            _cell.value = cell[item.valueField]
            _cell.label = cell[item.valueText]
            if (!_cell.label && _cell.label !== 0) return
          } else {
            _cell.$value = cell[item.cardValField]
            _cell = {..._cell, ...cell}
          }
  
          if (item.linkField) {
            _cell.ParentID = cell[item.linkField] === undefined ? '' : cell[item.linkField]
          }
          if (item.subFields) {
            item.subFields.forEach(m => {
              _cell[m.field] = cell[m.field] === undefined ? '' : cell[m.field]
            })
          }

          if (item.disableField && cell[item.disableField] && /^true$/ig.test(cell[item.disableField])) {
            _cell.$disabled = true
          }
  
          options.push(_cell)
        })

        item.oriOptions = [...item.oriOptions, ...options]

        // if (item.type === 'link') {
        if (item.linkField) {
          item.options = item.oriOptions.filter(option => option.ParentID === item.supInitVal || option.value === '')
        // } else if (['select', 'radio', 'checkbox', 'checkcard', 'multiselect'].includes(item.type)) {
        } else {
          item.options = item.oriOptions
        }

        if (['select', 'link', 'radio'].includes(item.type) && typeof(item.initval) === 'string' && item.initval.indexOf('$first') > -1) { // 选中第一项
          item.initval = item.options[0] ? item.options[0].value : ''
          this.record[item.field] = item.initval
        }

        if (item.subFields && item.options.length > 0) {
          let initval = item.initval
          if (item.type === 'cascader' && item.separator) {
            initval = initval.split(item.separator).pop()
          }
          // eslint-disable-next-line
          let option = item.options.filter(cell => initval == cell.value)[0]
  
          if (option) {
            reFieldsVal = reFieldsVal || {}
            item.subFields.forEach(n => {
              reFieldsVal[n.field] = option[n.field]
            })
          }
        }
      }
      
      return item
    })

    if (reFieldsVal) {
      _formlist = _formlist.map((cell, i) => {
        if (reFieldsVal[cell.field] === undefined) return cell

        cell.initval = reFieldsVal[cell.field]
        this.record[cell.field] = reFieldsVal[cell.field]

        setTimeout(() => {
          MKEmitter.emit('mkFC', 'input', cell.uuid, reFieldsVal[cell.field])
        }, i * 5)

        return cell
      })
    }

    this.setState({
      formlist: _formlist
    })
  }

  handleConfirmPassword = (rule, value, callback, item) => {
    let val = parseFloat(value)

    if (!isNaN(val)) {
      if (typeof(item.min) === 'number' && val < item.min) {
        callback(item.label + '最小值为 ' + item.min)
      } else if (typeof(item.max) === 'number' && val > item.max) {
        callback(item.label + '最大值为 ' + item.max)
      } else {
        callback()
      }
    } else {
      callback()
    }
  }

  recordChange = (values, item) => {
    this.record = {...this.record, ...values}

    if (!item || !item.controlFields) return

    let map = new Map()
    this.state.formlist.forEach(cell => {
      if (!cell.field) return
      map.set(cell.field, cell)
    })

    let reset = (current) => {
      let val = this.record[current.field]

      current.controlFields.forEach(cell => {
        let m = map.get(cell.field)
        m.hidden = current.hidden || !cell.values.includes(val)

        if (m.hidden) {
          m.initval = this.record[m.field]
        }

        map.set(cell.field, m)

        if (m.controlFields) {
          reset(m)
        }
      })
    }

    reset(item)

    this.setState({
      formlist: this.state.formlist.map(cell => {
        if (cell.field) {
          return map.get(cell.field)
        }
        return cell
      })
    })
  }

  getFields() {
    const { getFieldDecorator } = this.props.form
    const { formlist } = this.state

    const fields = []

    formlist.forEach((item, index) => {
      if (item.hidden) return

      if (item.type === 'split') {
        fields.push(
          <Col span={24} key={index}>
            <p className="mk-form-split-line" style={item.style}>{item.label}</p>
          </Col>
        )
      } else if (item.type === 'hint') {
        fields.push(
          <Col span={item.span || 24} key={index}>
            <Form.Item className="hint" colon={false} label={item.label ? <span className="mk-form-label" style={item.style}>{item.label}</span> : ' '} labelCol={item.labelCol} wrapperCol={item.wrapperCol}>
              <div className="message" style={item.style}>{item.message}</div>
            </Form.Item>
          </Col>
        )
      } else if (item.type === 'formula') {
        fields.push(
          <Col span={item.span || 24} key={index}>
            <Form.Item className="hint" colon={false} label={<span className="mk-form-label" style={item.style}>{item.label}</span>} labelCol={item.labelCol} wrapperCol={item.wrapperCol}>
              <MkFormula config={item} data={this.record}></MkFormula>
            </Form.Item>
          </Col>
        )
      } else {
        let content = null
        let className = ''
        let label = item.tooltip ? <Tooltip placement="topLeft" title={item.tooltip}><QuestionCircleOutlined style={{color: '#c49f47', marginRight: '3px'}}/>
          <span className="mk-form-label" style={item.style}>{item.label}</span>
        </Tooltip> : <span className="mk-form-label" style={item.style}>{item.label}</span>
      
        if (item.type === 'text' || item.type === 'linkMain') {
          content = (<MKInput config={item} onChange={(val, defer) => !defer && this.recordChange({[item.field]: val})} onSubmit={this.props.inputSubmit} />)
        } else if (item.type === 'number') {
          content = (<MKNumberInput config={item} onChange={(val, defer) => !defer && this.recordChange({[item.field]: val})} onSubmit={this.props.inputSubmit} />)
        } else if (item.type === 'select' || item.type === 'link' || item.type === 'multiselect') {
          content = (<MKSelect config={item} onChange={(val, other) => this.recordChange({[item.field]: val, ...other}, item)} onSubmit={this.props.inputSubmit} />)
        } else if (item.type === 'cascader') {
          content = (<MkCascader config={item} onChange={(val, other) => this.recordChange({[item.field]: val, ...other}, item)}/>)
        } else if (item.type === 'color') {
          content = (<MKColor config={item} onChange={(val) => this.recordChange({[item.field]: val})}/>)
        } else if (item.type === 'checkcard') {
          className = 'checkcard'
          content = (<MKCheckCard config={item} onChange={(val, other) => this.recordChange({[item.field]: val, ...other}, item)}/>)
        } else if (item.type === 'switch') {
          content = (<MKSwitch config={item} onChange={(val) => this.recordChange({[item.field]: val}, item)}/>)
        } else if (item.type === 'checkbox') {
          content = (<MKCheckbox config={item} onChange={(val) => this.recordChange({[item.field]: val})}/>)
        } else if (item.type === 'radio') {
          content = (<MKRadio config={item} onChange={(val, other) => this.recordChange({[item.field]: val, ...other}, item)}/>)
        } else if (item.type === 'date' || item.type === 'datemonth') {
          content = (<MKDatePicker config={item} onChange={(val) => this.recordChange({[item.field]: val})} />)
        } else if (item.type === 'fileupload') {
          className = item.readonly ? 'readonly' : ''
          className += item.fileType === 'picture-card' ? ' file-upload' : ''
          content = (<MKFileUpload config={item} onChange={(val) => this.recordChange({[item.field]: val})} />)
        } else if (item.type === 'textarea') {
          content = (<MKTextArea config={item} onChange={(val, defer) => !defer && this.recordChange({[item.field]: val})}/>)
        } else if (item.type === 'rate') {
          content = (<Rate count={item.rateCount} disabled={item.readonly} onChange={(val) => this.recordChange({[item.field]: val})} character={item.character ? <MkIcon type={item.character}/> : <StarFilled />} allowHalf={item.allowHalf}/>)
        } else if (item.type === 'brafteditor') {
          content = (<MKEditor config={item} onChange={(val) => this.recordChange({[item.field]: val})}/>)
          label = item.hidelabel !== 'true' ? label : ''
        }

        if (!content) return

        fields.push(
          <Col span={item.span || 24} key={index}>
            <Form.Item
              label={label}
              colon={false}
              className={className}
              extra={item.extra || null}
              labelCol={item.labelCol}
              wrapperCol={item.wrapperCol}
            >
              {getFieldDecorator(item.field, {
                initialValue: item.initval,
                rules: item.rules
              })(content)}
            </Form.Item>
          </Col>
        )
      }
    })
    
    return fields
  }

  handleConfirm = () => {
    const { formlist } = this.state

    // 表单提交时检查输入值是否正确
    return new Promise((resolve, reject) => {
      this.props.form.validateFieldsAndScroll((err, values) => {
        if (err) {
          reject(err)
          return
        }
        let search = []
        let record = {...this.record, ...values}

        formlist.forEach(item => {
          if (!item.field) return
    
          let _item = {
            type: item.type,
            readonly: item.readonly,
            readin: item.readin,
            writein: item.writein,
            fieldlen: item.fieldlength,
            key: item.field
          }
    
          _item.value = record[item.field] !== undefined ? record[item.field] : ''
    
          if (item.type === 'funcvar') {
            _item.value = ''
          } else if (_item.value && (item.type === 'text' || item.type === 'textarea' || item.type === 'linkMain') && typeof(_item.value) === 'string') {
            _item.value = _item.value.replace(/\t*|\v*/g, '')       // 去除制表符
    
            if (item.interception === 'true') {           // 去除首尾空格
              _item.value = _item.value.replace(/(^\s*|\s*$)/g, '')
            }
            if (item.type === 'text') { // 特殊字段替换
              _item.value = _item.value.replace(/^(\s*)@appkey@(\s*)$/ig, window.GLOB.appkey)
              _item.value = _item.value.replace(/^(\s*)@SessionUid@(\s*)$/ig, (localStorage.getItem('SessionUid') || ''))
              _item.value = _item.value.replace(/^(\s*)@bid@(\s*)$/ig, (this.props.BID || ''))
            }
          } else if (item.type.indexOf('date') > -1) {
            if (item.declareType === 'nvarchar(50)') {
              _item.type = 'text'
            }
          } else if (item.declare === 'decimal' && ['select', 'link', 'radio', 'checkcard'].includes(item.type)) {
            _item.type = 'number'
            _item.fieldlen = item.decimal || 0
          }

          if (item.type === 'text' && item.lenControl && item.lenControl !== 'limit') {
            if (item.lenControl === 'left') {
              _item.value = _item.value.substr(0, item.fieldlength)
            } else {
              _item.value = _item.value.slice(-item.fieldlength)
            }
          }
    
          search.push(_item)
        })

        resolve(search)
      })
    })
  }

  render() {
    const { action } = this.props

    let _align = 'left_right'
    if (action.setting && action.setting.align) {
      _align = action.setting.align
    }
    let space = ' space-normal'
    if (action.setting && action.setting.verticalSpace) {
      space = ' space-' + action.setting.verticalSpace
    }

    return (
      <Form className={'main-form-field ' + _align + space}>
        <Row gutter={24}>{this.getFields()}</Row>
      </Form>
    )
  }
}

export default Form.create()(MainSearch)